﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Data.Common
Imports System.Data.Entity
Imports System.Data.Entity.Infrastructure.Interception
Imports System.Data.Entity.SqlServer
Imports System.Data.SqlClient
Imports System.Diagnostics
Imports System.Reflection
Imports System.Linq

Namespace Log1

    Public Class InterceptorLogging
        Inherits DbCommandInterceptor

        Private _logger As ILogger = New Logger1()

        Private ReadOnly _stopwatch As Stopwatch = New Stopwatch()

        Public Overrides Sub ScalarExecuting(ByVal command As DbCommand, ByVal interceptionContext As DbCommandInterceptionContext(Of Object))
            MyBase.ScalarExecuting(command, interceptionContext)
            _stopwatch.Restart()
        End Sub

        Public Overrides Sub ScalarExecuted(ByVal command As DbCommand, ByVal interceptionContext As DbCommandInterceptionContext(Of Object))
            _stopwatch.[Stop]()
            If interceptionContext.Exception IsNot Nothing Then
                _logger.[Error](interceptionContext.Exception, "Error executing command: {0}", command.CommandText)
            Else
                _logger.TraceApi("SQL Database", "Interceptor.ScalarExecuted", _stopwatch.Elapsed, "Command: {0}: ", command.CommandText)
            End If

            MyBase.ScalarExecuted(command, interceptionContext)
        End Sub

        Public Overrides Sub NonQueryExecuting(ByVal command As DbCommand, ByVal interceptionContext As DbCommandInterceptionContext(Of Integer))
            MyBase.NonQueryExecuting(command, interceptionContext)
            _stopwatch.Restart()
        End Sub

        Public Overrides Sub NonQueryExecuted(ByVal command As DbCommand, ByVal interceptionContext As DbCommandInterceptionContext(Of Integer))
            _stopwatch.[Stop]()
            If interceptionContext.Exception IsNot Nothing Then
                _logger.[Error](interceptionContext.Exception, "Error executing command: {0}", command.CommandText)
            Else
                _logger.TraceApi("SQL Database", "Interceptor.NonQueryExecuted", _stopwatch.Elapsed, "Command: {0}: ", command.CommandText)
            End If

            MyBase.NonQueryExecuted(command, interceptionContext)
        End Sub

        Public Overrides Sub ReaderExecuting(ByVal command As DbCommand, ByVal interceptionContext As DbCommandInterceptionContext(Of DbDataReader))
            MyBase.ReaderExecuting(command, interceptionContext)
            _stopwatch.Restart()
        End Sub

        Public Overrides Sub ReaderExecuted(ByVal command As DbCommand, ByVal interceptionContext As DbCommandInterceptionContext(Of DbDataReader))
            _stopwatch.[Stop]()
            If interceptionContext.Exception IsNot Nothing Then
                _logger.[Error](interceptionContext.Exception, "Error executing command: {0}", command.CommandText)
            Else
                _logger.TraceApi("SQL Database", "Interceptor.ReaderExecuted", _stopwatch.Elapsed, "Command: {0}: ", command.CommandText)
            End If

            MyBase.ReaderExecuted(command, interceptionContext)
        End Sub
    End Class

End Namespace